@@ -59,7 +59,7 @@ struct getdns_list
 	struct mem_funcs mf;
 };
 
-inline static getdns_list *_getdns_list_create_with_mf(struct mem_funcs *mf)
+static inline getdns_list *_getdns_list_create_with_mf(struct mem_funcs *mf)
 { return getdns_list_create_with_extended_memory_functions(
          mf->mf_arg, mf->mf.ext.malloc, mf->mf.ext.realloc, mf->mf.ext.free); }
 
