@@ -68,7 +68,8 @@
 #define STUB_TCP_ERROR -2
 
 /* Don't currently have access to the context whilst doing handshake */
-#define TIMEOUT_TLS 2500
+#define MIN_TLS_HS_TIMEOUT 2500
+#define MAX_TLS_HS_TIMEOUT 7500
 /* Arbritray number of message for EDNS keepalive resend*/
 #define EDNS_KEEPALIVE_RESEND 5
 
@@ -241,7 +242,7 @@ match_edns_opt_rr(uint16_t code, uint8_t *response, size_t response_len,
 	uint8_t *data = (uint8_t *)rr_iter->pos;
 	size_t data_len = rr_iter->nxt - rr_iter->pos;
 	(void) gldns_wire2str_rr_scan(
-	    &data, &data_len, &str, &str_len, (uint8_t *)rr_iter->pkt, rr_iter->pkt_end - rr_iter->pkt);
+	    &data, &data_len, &str, &str_len, (uint8_t *)rr_iter->pkt, rr_iter->pkt_end - rr_iter->pkt, NULL);
 	DEBUG_STUB("%s %-35s: OPT RR: %s",
 	           STUB_DEBUG_READ, __FUNC__, str_spc);
 #endif
@@ -455,7 +456,8 @@ tcp_connect(getdns_upstream *upstream, getdns_transport_list_t transport)
 		    GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_WARNING,
 		    "%-40s : Upstream   : Could not setup TCP TFO\n",
 		     upstream->addr_str);
-
+#  else
+	(void)transport;
 #  endif/* HAVE_DECL_TCP_FASTOPEN*/
 # endif	/* HAVE_DECL_TCP_FASTOPEN_CONNECT */
 #endif	/* USE_OSX_TCP_FASTOPEN */
@@ -980,13 +982,23 @@ tls_do_handshake(getdns_upstream *upstream)
 	int r;
 	while ((r = _getdns_tls_connection_do_handshake(upstream->tls_obj)) != GETDNS_RETURN_GOOD)
 	{
+		uint64_t timeout_tls = _getdns_ms_until_expiry(upstream->expires);
+
+		if (timeout_tls < MIN_TLS_HS_TIMEOUT)
+			timeout_tls = MIN_TLS_HS_TIMEOUT;
+		else if (timeout_tls > MAX_TLS_HS_TIMEOUT)
+			timeout_tls = MAX_TLS_HS_TIMEOUT;
+
+		DEBUG_STUB("%s %-35s: FD:  %d, do_handshake -> %d (timeout: %d)\n",
+		    STUB_DEBUG_SETUP_TLS, __FUNC__, upstream->fd, r, (int)timeout_tls);
+
 		switch (r) {
 			case GETDNS_RETURN_TLS_WANT_READ:
 				GETDNS_CLEAR_EVENT(upstream->loop, &upstream->event);
 				upstream->event.read_cb = upstream_read_cb;
 				upstream->event.write_cb = NULL;
 				GETDNS_SCHEDULE_EVENT(upstream->loop,
-				    upstream->fd, TIMEOUT_TLS, &upstream->event);
+				    upstream->fd, timeout_tls, &upstream->event);
 				upstream->tls_hs_state = GETDNS_HS_READ;
 				return STUB_TCP_RETRY;
 			case GETDNS_RETURN_TLS_WANT_WRITE:
@@ -994,7 +1006,7 @@ tls_do_handshake(getdns_upstream *upstream)
 				upstream->event.read_cb = NULL;
 				upstream->event.write_cb = upstream_write_cb;
 				GETDNS_SCHEDULE_EVENT(upstream->loop,
-				    upstream->fd, TIMEOUT_TLS, &upstream->event);
+				    upstream->fd, timeout_tls, &upstream->event);
 				upstream->tls_hs_state = GETDNS_HS_WRITE;
 				return STUB_TCP_RETRY;
 			default:
@@ -1198,7 +1210,12 @@ stub_tls_write(getdns_upstream *upstream, getdns_tcp_state *tcp,
 	_getdns_tls_connection* tls_obj = upstream->tls_obj;
 	uint16_t        padding_sz;
 
-	int q = tls_connected(upstream);
+	int q;
+       
+	if (netreq->owner->expires > upstream->expires)
+		upstream->expires = netreq->owner->expires;
+
+	q = tls_connected(upstream);
 	if (q != 0)
 		return q;
 	/* This is the case where the upstream is connected but it isn't an authenticated
@@ -2225,8 +2242,16 @@ upstream_schedule_netreq(getdns_upstream *upstream, getdns_network_req *netreq)
 			/* Set a timeout on the upstream so we can catch failed setup*/
 			upstream->event.timeout_cb = upstream_setup_timeout_cb;
 			GETDNS_SCHEDULE_EVENT(upstream->loop, upstream->fd,
-			    _getdns_ms_until_expiry(netreq->owner->expires)/2,
+			    _getdns_ms_until_expiry(netreq->owner->expires)/5*4,
 			    &upstream->event);
+#if defined(HAVE_DECL_TCP_FASTOPEN) && HAVE_DECL_TCP_FASTOPEN \
+ && !(defined(HAVE_DECL_TCP_FASTOPEN_CONNECT) && HAVE_DECL_TCP_FASTOPEN_CONNECT) \
+ && !(defined(HAVE_DECL_MSG_FASTOPEN) && HAVE_DECL_MSG_FASTOPEN)
+			if (upstream->transport == GETDNS_TRANSPORT_TCP) {
+				/* Write immediately! */
+				upstream_write_cb(upstream);
+			}
+#endif
 		} else {
 			GETDNS_SCHEDULE_EVENT(upstream->loop,
 			    upstream->fd, TIMEOUT_FOREVER, &upstream->event);
@@ -2292,78 +2317,14 @@ _getdns_submit_stub_request(getdns_network_req *netreq, uint64_t *now_ms)
 
 	case GETDNS_TRANSPORT_TLS:
 	case GETDNS_TRANSPORT_TCP:
-		upstream_schedule_netreq(netreq->upstream, netreq);
-		/* For TLS, set a short timeout to catch setup problems. This is reset
-		   when the connection is successful.*/
 		GETDNS_CLEAR_EVENT(dnsreq->loop, &netreq->event);
-		/*************************************************************
-		 ******                                                  *****
-		 ******            Scheduling differences of             *****
-		 ******      synchronous and asynchronous requests       *****
-		 ******                                                  *****
-		 *************************************************************
-		 *
-		 * Besides the asynchronous event loop, which is typically
-		 * shared with the application, every getdns context also
-		 * has another event loop (not registered by the user) which
-		 * is used specifically and only for synchronous requests:
-		 * context->sync_eventloop.
-		 *
-		 * We do not use the asynchronous loop for the duration of the
-		 * synchronous query, because:
-		 * - Callbacks for outstanding (and thus asynchronous) queries
-		 *   might fire as a side effect.
-		 * - But worse, since the asynchronous loop is created and 
-		 *   managed by the user, which may well have her own non-dns
-		 *   related events scheduled against it, they will fire as
-		 *   well as a side effect of doing the synchronous request!
-		 *
-		 *
-		 * Transports that keep connections open, have their own event
-		 * structure to keep their connection state.  The event is 
-		 * associated with the upstream struct.  Note that there is a
-		 * separate upstream struct for each state full transport, so
-		 * each upstream has multiple transport structs!
-		 *
-		 *     side note: The upstream structs have their own reference
-		 *                to the "context's" event loop so they can,
-		 *                in theory, be detached (to finish running 
-		 *                queries for example).
-		 *
-		 * If a synchronous request is scheduled for such a transport,
-		 * then the sync-loop temporarily has to "run" that 
-		 * upstream/transport's event!  Outstanding requests for that
-		 * upstream/transport might come in while processing the 
-		 * synchronous call.  When this happens, they are queued up
-		 * (at upstream->finished_queue) and an timeout event of 1
-		 * will be scheduled against the asynchronous loop to start
-		 * processing those received request as soon as the 
-		 * asynchronous loop will be run.
-		 *
-		 *
-		 * When getdns is linked with libunbound 1.5.8 or older, then
-		 * when a RECURSING synchronous request is made then 
-		 * outstanding asynchronously scheduled RECURSING requests
-		 * may fire as a side effect, as we reuse the same code path 
-		 * For both synchronous and asynchronous calls,
-		 * ub_resolve_async() is used under the hood.
-		 *
-		 * With libunbound versions newer than 1.5.8, libunbound will
-		 * share the event loops used with getdns which will prevent
-		 * these side effects from happening.
-		 *
-		 *
-		 * The event loop used for a specific request is in 
-		 * dnsreq->loop.  The asynchronous is always also available
-		 * at the upstream as upstream->loop. 
-		 */
 		GETDNS_SCHEDULE_EVENT(
 		    dnsreq->loop, -1,
 		    _getdns_ms_until_expiry2(dnsreq->expires, now_ms),
 		    getdns_eventloop_event_init(
 		    &netreq->event, netreq, NULL, NULL,
 		    stub_timeout_cb));
-
+		upstream_schedule_netreq(netreq->upstream, netreq);
 		return GETDNS_RETURN_GOOD;
 	default:
 		return GETDNS_RETURN_GENERIC_ERROR;
