@@ -1,6 +1,6 @@
 /**
  * \file
- * \brief Public interfaces to getdns, include in your application to use getdns API.
+ * \brief Testing interfaces to getdns
  *
  * This source was taken from the original pseudo-implementation by
  * Paul Hoffman.
@@ -37,11 +37,19 @@
 
 #include "getdns/getdns_ext_libuv.h"
 #include <uv.h>
+#ifdef __clang__
+#pragma clang diagnostic push
+#pragma clang diagnostic ignored "-Wgnu-zero-variadic-macro-arguments"
+#endif
 #include <check.h>
+#ifdef __clang__
+#pragma clang diagnostic pop
+#endif
 #include "check_getdns_common.h"
 
 void run_event_loop_impl(struct getdns_context* context, void* eventloop) {
     uv_loop_t* loop = (uv_loop_t*) eventloop;
+    (void)context;
     uv_run(loop, UV_RUN_DEFAULT);
 }
 
