@@ -28,6 +28,7 @@
 #include "config.h"
 #include <getdns/getdns.h>
 #include <getdns/getdns_extra.h>
+#include <stdint.h>
 #include <string.h>
 #include <errno.h>
 #include <limits.h>
@@ -36,10 +37,17 @@
 #include <shlobj.h>
 #else
 #include <pwd.h>
-#include <unistd.h>
 #endif
 #include <signal.h>
 #include <limits.h>
+#ifndef HAVE_GETOPT
+#include "getopt.h"
+#else
+#include <unistd.h>
+#endif
+#if defined(ENABLE_SYSTEMD)
+#include <systemd/sd-daemon.h>
+#endif
 
 #ifdef HAVE_GETDNS_YAML2DICT
 getdns_return_t getdns_yaml2dict(const char *str, getdns_dict **dict);
@@ -344,7 +352,7 @@ static getdns_return_t parse_config_file(const char *fn)
 typedef struct dns_msg {
 	getdns_transaction_t  request_id;
 	getdns_dict          *request;
-	uint32_t              rt;
+	getdns_resolution_t   rt;
 	uint32_t              ad_bit;
 	uint32_t              do_bit;
 	uint32_t              cd_bit;
@@ -807,6 +815,9 @@ main(int argc, char **argv)
 		}
 	}
 
+	stubby_local_log(NULL,GETDNS_LOG_UPSTREAM_STATS, GETDNS_LOG_INFO,
+		       "Stubby version: %s\n", STUBBY_PACKAGE_STRING);
+
 	if ((r = getdns_context_create(&context, 1))) {
 		fprintf(stderr, "Create context failed: %s\n",
 		        _getdns_strerror(r));
@@ -970,6 +981,9 @@ main(int argc, char **argv)
 		} else {
 #ifdef SIGPIPE
 			(void)signal(SIGPIPE, SIG_IGN);
+#endif
+#ifdef ENABLE_SYSTEMD
+			sd_notifyf(0, "READY=1\nMAINPID=%u", getpid());
 #endif
 			getdns_context_run(context);
 		}
@@ -1017,6 +1031,9 @@ main(int argc, char **argv)
 			       "Starting DAEMON....\n");
 #ifdef SIGPIPE
 		(void)signal(SIGPIPE, SIG_IGN);
+#endif
+#ifdef ENABLE_SYSTEMD
+		sd_notify(0, "READY=1");
 #endif
 		getdns_context_run(context);
 	}
